# For now, the User model is being persisted in two repositories:
#   * On postgres, using ActiveRecord
#   * OrientDb as graph nodes
#
# Goals: Keep the users in both (postgres, orientdb)
#
#               Devise Controllers -----> User model (with active record api)
#              /
#     Client -/
#             \
#              \
#               Portal Mec Controllers -> User repository
#
# How works?
#   For keep simple, user node in orientdb stored only a reference to the real user in postgres.
#   So, the user node will be only created or destroyed.
#
#    When devise create user, the active record hooks will sync with user repository
#    And, when the application call the user repository, it will keep sync with active record user model
class User < ActiveRecord::Base
  # Include default devise modules. Others available are:
  # :confirmable, :lockable, :timeoutable and :omniauthable
  devise :database_authenticatable, :registerable,
         :recoverable, :rememberable, :trackable, :validatable
  has_and_belongs_to_many :roles

  after_create SyncUserRepositoryService.new
  after_destroy SyncUserRepositoryService.new

  def graph_id
    @graph_id ||= user_repository.get_graph_id(self)
  end

  private

  def user_repository
    Portalmec::Application.repository.for :user
  end

end
